/*********************************************************************

  Filename:  clock.cpp
  Chapter:   C      Appendix C - C++ Language Guide
  Section:   C.10   Functions - Operator Overloading
  Compiler:  Borland C++     Version 3.1       Summer 1993

  C++ for C Programmers, Edition 2     By Ira Pohl

*********************************************************************/

//Clock to show overloading
//  Title: clock

#include <iostream.h>

class clock {
public:
   clock(unsigned long int i); //constructor and conversion
   void print();               //formatted printout
   void tick();                //add one second
   clock operator ++() { this -> tick(); return (*this); }
   void reset (clock& c);
   friend clock operator +(clock c1, clock c2);
   clock operator -(clock c);
   friend clock operator *(unsigned long int m, clock c);
   friend clock operator *(clock c, unsigned long int m);
private:
   unsigned long int  tot_secs, secs, mins, hours, days;
};

inline clock::clock(unsigned long int i)
{
   tot_secs = i;
   secs = tot_secs % 60;
   mins = (tot_secs / 60) % 60;
   hours = (tot_secs / 3600) % 24;
   days = tot_secs / 86400;
}

void clock::tick()
{
   clock  temp = clock(++tot_secs);

   secs = temp.secs;
   mins = temp.mins;
   hours = temp.hours;
   days = temp.days;
}

void clock::reset(clock& c)
{
   tot_secs = c.tot_secs;
   secs = c.secs;
   mins = c.mins;
   hours = c.hours;
   days = c.days;
}

clock operator +(clock c1, clock c2)
{
   unsigned long int total_sec = c1.tot_secs + c2.tot_secs;
   clock temp = total_sec;
   return (temp);
}

clock clock::operator -(clock c)
{
   unsigned long int total_sec = tot_secs - c.tot_secs;
   clock temp = total_sec;
   return (temp);
}

clock operator *(unsigned long int m, clock c)
{
   unsigned long int total_sec = m * c.tot_secs;
   clock temp = total_sec;
   return (temp);
}

clock operator *(clock c, unsigned long int m)
{
    unsigned long int total_sec = m * c.tot_secs;
   clock temp = total_sec;
   return (temp);
}

void clock::print()
{
   cout << days << " d :" << hours << " h :"
	<< mins << " m :" << secs << " s\n";
}

main()
{
   clock t1(59), t2(172799);  //min - 1 sec and 2 days - 1 sec
   clock t3(0);
   clock c1(900), c2(400);

   cout << "initial times are\n";
   t1.print();
   t2.print();
   ++t1;  ++t2;          //t1++; t2++ are also possible
   cout << "after one second times are\n";
   t1.print();
   t2.print();
   cout << "\nt1 + t2   t1 * 5    6 * t1 * 5     6 * t1 * 5 - t2\n";
   t3 = t1 + t2;
   t3.print();
   t3 = t1 * 5;
   t3.print();
   t3 = 6 * t3;
   t3.print();
   t3 = t3 - t1;
   t3.print();
   c1.reset(c2);
   c2.reset(100);
   cout << "\nc1 and c2\n";
   c1.print();
   c2.print();
   return (0);
}