/*********************************************************************

  Filename:  vect.cpp
  Chapter:   C      Appendix C - C++ Language Guide
  Section:   C.9    Classes - Inheritance
  Compiler:  Borland C++     Version 3.1       Summer 1993

  C++ for C Programmers, Edition 2     By Ira Pohl

*********************************************************************/

//Implementation of a safe array type vect
//  Title: vect

#include <iostream.h>
#include <stdlib.h>

class vect {
public:
   //constructors and destructor
   vect() { size = 10; p = new int[10];} //create a size 10 array
   vect(int n);                         //create a size n array
   vect(vect& v);                       //initialization by vect
   vect(int a[], int n);                //initialization by array
   ~vect() { delete [] p; }             //destructor
   //other member functions
   int  ub() { return (size - 1); }     //upper bound
   int& operator [](int i);             //obtain range checked element
   vect& operator =(const vect& v);
   vect operator +(const vect& v);
private:
   int* p;                              //base pointer
   int size;                            //number of elements
};

vect::vect(int n)
{
   if (n <= 0) {
      cerr << "illegal vect size: " << n << endl;
      exit(1);
   }
   size = n;
   p = new int[size];
}

int& vect::operator [](int i)
{
   if (i < 0 || i > (size - 1) ) {
      cerr << "illegal vect index: " << i << endl;
      exit(1);
   }
   return (p[i]);
}


main()
{
   vect   v1, v2(10), v3;
   int i, j;

   for (i = 0; i <= v1.ub(); ++i)
      v1[i] = ((i + 5) * 10);

   cout << "\nvector v1 is ";
   for (i = 0; i <= v1.ub(); ++i)
      cout << "[" << i << "] = " << v1[i] << "\t";
   cout << endl;

   for (i = 0; i <= v2.ub(); ++i)
      v2[i] = (i + i);

   cout << "\nvector v2 is ";
   for (i = 0; i <= v2.ub(); ++i)
      cout << "[" << i << "] = " << v2[i] << "\t";
   cout << endl;
   return (0);
}