/*********************************************************************

  Filename:  vect_bnd.cpp
  Chapter:   C      Appendix C - C++ Language Guide
  Section:   C.9    Classes - Inheritance
  Compiler:  Borland C++     Version 3.1       Summer 1993

  C++ for C Programmers, Edition 2     By Ira Pohl

*********************************************************************/

//Implementation of a safe array type vect
//  Title: vect_bounds

#include <iostream.h>
#include <stdlib.h>

class vect {
public:
   //constructors and destructor
   vect();                         //create a size 10 array
   vect(int l);                    //create a size l array
   vect(const vect& v);            //initialization by vect
   vect(const int a[], int l);     //initialization by array
   ~vect(){ delete []p;}
   int  ub() { return (size - 1);} //upper bound
   int& operator [](int i);        //obtain range checked element
   vect& operator =(const vect& v);
   vect operator +(const vect& v);
protected:
   int *p;                         //base pointer
   int size;                       //number of elements
};

vect::vect()
{
   size = 10;
   p = new int[size];
}

vect::vect(int n)
{
   if (n <= 0) {
      cerr << "illegal vect size: " << n << endl;
      exit(1);
   }
   size = n;
   p = new int[size];
}

vect::vect(const int a[], int n)
{
   if (n <= 0) {
      cerr << "illegal vect size: " << n << endl;
      exit(1);
   }
   size = n;
   p = new int[size];
   for (int i = 0; i < size; ++i)
      p[i] = a[i];
}

vect::vect(const vect& v)
{
   size = v.size;
   p = new int[size];
   for (int i = 0; i < size; ++i)
      p[i] = v.p[i];
}

int& vect::operator [](int i)
{
   if (i < 0 || i > (size - 1) ) {
      cerr << "illegal vect index: " << i << endl;
      exit(1);
   }
   return (p[i]);
}

vect& vect::operator =(const vect& v)
{
   int s = (size < v.size) ? size : v.size;

   if (v.size != size)
      cerr << "copying different size arrays "
           << size << " and " << v.size << endl;
   for (int i = 0; i < s; ++i)
      p[i] = v.p[i];
   return (*this);
}

vect vect::operator +(const vect& v)
{
   int s = (size < v.size) ? size : v.size;
   vect  sum(s);

   if (v.size != size)
      cerr << "adding different size arrays "
	   << size << " and " << v.size << endl;
   for (int i = 0; i < s; ++i)
      sum.p[i] = p[i] + v.p[i];
   return (sum);
}

//Derived safe array type vect_bnd
class vect_bnd : public vect {
public:
   vect_bnd();
   vect_bnd(int, int);
   int& operator[](int);
   int  ub() { return (u_bnd); }
   int  lb() { return (l_bnd); }
private:
   int l_bnd, u_bnd;
};

vect_bnd::vect_bnd() : vect(10)
{
   l_bnd = 0;
   u_bnd = 9;
}

vect_bnd::vect_bnd(int lb, int ub) : vect(ub - lb + 1)
{
   l_bnd = lb;
   u_bnd = ub;
}

int& vect_bnd::operator[](int i)
{
    if (i < l_bnd || u_bnd < i) {
	cerr<<"index out of range\n";
	exit(1);
    };
    return  (p[i - l_bnd]);
}

main()
{
   vect_bnd   v1, v2(4, 10), v3;
   int i, j;

   for (i = v1.lb(); i <= v1.ub(); ++i)
      v1[i] = ((i + 5) * 10);

   cout << "\nvector v1 is ";
   for (i = v1.lb(); i <= v1.ub(); ++i)
      cout << "[" << i << "] = " << v1[i] << "\t";
   cout << endl;

   for (i = v2.lb(); i <= v2.ub(); ++i)
      v2[i] = (i + i);

   cout << "\nvector v2 is ";
   for (i = v2.lb(); i <= v2.ub(); ++i)
      cout << "[" << i << "] = " << v2[i] << "\t";
   cout << endl;
   v3 = v1;

   cout << "\nvector v3 is ";
   for (i = v3.lb(); i <= v3.ub(); ++i)
      cout << "[" << i << "] = " << v3[i] << "\t";
   cout << endl;
   return (0);
}