/*********************************************************************

  Filename:  mpy.cpp
  Chapter:   5      Operator Overloading and Conversions
  Section:   5.4    Friend Functions
  Compiler:  Borland C++     Version 3.1       Summer 1993

  C++ for C Programmers, Edition 2     By Ira Pohl

*********************************************************************/

//A safe array type vect with [] overloaded
//with matrix added
//  Title: mpy

#include <iostream.h>
#include <stdlib.h>               //for exit

class matrix;                  //forward reference

class vect {
public:
   //constructors and destructor
   vect();                           //create a size 10 array
   vect(int n);                      //create a size n array
   vect(const vect& v);              //initialization by vect
   vect(const int a[], int n);       //initialization by array
   ~vect() { delete p; }
   //other member functions
   int  ub() { return (size - 1); }  //upper bound
   int& operator [](int i);          //range checked element
   vect& operator =(const vect& v);  //overload assignment
   vect  operator +(const vect& v);  //overload plus
   void print();                     //print vector
private:
   int* p;                        //base pointer
   int size;                      //number of elements
   friend vect mpy(const vect& v, const matrix& m);
};

vect::vect()
{
   size = 10;
   p = new int[size];
}

vect::vect(int n)
{
   if (n <= 0) {
      cerr << "illegal vect size: " << n << endl;
      exit(1);
   }
   size = n;
   p = new int[size];
}

vect::vect(const int a[], int n)
{
   if (n <= 0) {
      cerr << "illegal vect size: " << n << endl;
      exit(1);
   }
   size = n;
   p = new int[size];
   for (int i = 0; i < size; ++i)
      p[i] = a[i];
}

vect::vect(const vect& v)
{
   size = v.size;
   p = new int[size];
   for (int i = 0; i < size; ++i)
      p[i] = v.p[i];
}

int& vect::operator [](int i)
{
   if (i < 0 || i > (size - 1) ) {
      cerr << "illegal vect index: " << i << endl;
      exit(1);
   }
   return (p[i]);
}

vect& vect::operator =(const vect& v)
{
   int s = (size < v.size) ? size : v.size;

   if (v.size != size)
      cerr << "copying different size arrays "
           << size << " and " << v.size << endl;
   for (int i = 0; i < s; ++i)
      p[i] = v.p[i];
   return (*this);
}

vect vect::operator +(const vect& v)
{
   int s = (size < v.size) ? size : v.size;
   vect  sum(s);

   if (v.size != size)
      cerr << "adding different size arrays "
	   << size << " and " << v.size << endl;
   for (int i = 0; i < s; ++i)
      sum.p[i] = p[i] + v.p[i];
   return (sum);
}

void vect::print()
{
   for (int i = 0; i <= (size-1); ++i)
      cout << p[i] << "\t";
   cout << endl;
}

class matrix {  //stores integer elements
private:
   int**     base;
   int       s1, s2;
   friend    vect mpy(const vect& v, const matrix& m);
public:
  int  ub1, ub2;
   matrix(int l1, int l2);
   ~matrix() { for (int i = 0; i <= ub1; ++i)
                 delete base[i];
              delete base; }
   int& element(int i, int j);
   void print();
};

vect mpy(const vect& v, const matrix& m)
{
   if (v.size != m.s1) {     //incorrect sizes
      cerr << "multiply failed - sizes incorrect "
	   << v.size << " and " << m.s1 << "\n";
      exit(1);
   }

   //use privileged access to p in vect and base in matrix
   vect ans(m.s2);
   int  i, j;
   for (i = 0; i <= m.ub2; ++i) {
      ans.p[i] = 0;
      for (j = 0; j <= m.ub1; ++j)
	 ans.p[i] += v.p[j] * m.base[j][i];
   }
   return (ans);
}

void matrix::print()
{
   int j;
   for (int i = 0; i <= ub1; ++i) {
      cout << "\nrow  " << i + 1 << endl;
      for (j = 0; j <= ub2; ++j)
         cout << base[i][j] << "\t";
   }
   cout << endl;
}


matrix::matrix(int l1, int l2)
{
   if (l1 < 0 || l2 < 0) {
      cerr << "illegal matrix size\n";
      exit(1);
   }
   s1 = l1;
   s2 = l2;
   base = new int*[s1];
   for (int i = 0; i < s1; ++i)
      base[i] = new int[s2];
   ub1 = s1 - 1;
   ub2 = s2 - 1;
};

int& matrix::element(int i, int j)
{
   if (i < 0 || i > ub1 || j < 0 || j > ub2) {
      cerr << "illegal matrix index\n";
      exit(1);
   }
   return (base[i][j]);
}

void init_vect(vect& v, int start, int incr)
{
   for (int i = 0; i <= v.ub(); ++i) {
      v[i] = start;
      start += incr;
   }
}

main()
{
   vect   a(3), b(3), c(6), d(6);
   matrix m(3, 6), mm(3, 6);
   int    i, j;

   a[0] = 1 + (a[1] = 1 + ( a[2] = 1)) ;
   init_vect(b, 1, 1);
   init_vect(c, 10, 10);
   init_vect(d, 100, 1);

   cout << "vector a is\n";
   a.print();
   cout << "\nvector b is\n";
   b.print();
   cout << "\nvector c is\n";
   c.print();
   cout << "\nvector d is\n";
   d.print();

   a = b = c;
   a = b + (c = a) + d;
   cout << "vector a is\n";
   a.print();
   cout << "\nvector b is\n";
   b.print();
   cout << "\nvector c is\n";
   c.print();
   cout << "\nvector d is\n";
   d.print();
   for (i = 0; i <= m.ub1; ++i)
      for (j = 0; j <= m.ub2; ++j)
         m.element(i,j) = mm.element(i,j) = i + j;

   cout << "\nmatrix m is\n";
   m.print();

   c = mpy(a, m);
   cout << "\nvector c product is\n";
   c.print();
   return (0);
}
