/*********************************************************************

  Filename:  safearay.cpp
  Chapter:   5      Operator Overloading and Conversions
  Section:   5.8    Overloading Assignment and Subscript Operators
  Compiler:  Borland C++     Version 3.1       Summer 1993

  C++ for C Programmers, Edition 2     By Ira Pohl

*********************************************************************/

//A safe array type vect with [] overloaded
//  Title: safe_array

#include <iostream.h>
#include <stdlib.h>               //for exit

class vect {
public:
   //constructors and destructor
   vect();                           //create a size 10 array
   vect(int n);                      //create a size n array
   vect(const vect& v);              //initialization by vect
   vect(const int a[], int n);       //initialization by array
   ~vect() { delete [] p; }
   //other member functions
   int  ub() { return (size - 1); }  //upper bound
   int& operator [](int i) const;    //range checked element
   vect& operator =(const vect& v);  //overload assignment
   vect  operator +(const vect& v);  //overload plus
   void print();                     //print vector
private:
   int* p;                        //base pointer
   int size;                      //number of elements
};

vect::vect()
{
   size = 10;
   p = new int[size];
}

vect::vect(int n)
{
   if (n <= 0) {
      cerr << "illegal vect size: " << n << "\n";
      exit(1);
   }
   size = n;
   p = new int[size];
}

vect::vect(const int a[], int n)
{
   if (n <= 0) {
      cerr << "illegal vect size: " << n << "\n";
      exit(1);
   }
   size = n;
   p = new int[size];
   for (int i = 0; i < size; ++i)
      p[i] = a[i];
}

vect::vect(const vect& v)
{
   size = v.size;
   p = new int[size];
   for (int i = 0; i < size; ++i)
      p[i] = v.p[i];
}

int& vect::operator [](int i) const
{
   if (i < 0 || i > ub() ) {
      cerr << "illegal vect index: " << i << "\n";
      exit(1);
   }
   return (p[i]);
}

vect& vect::operator =(const vect& v)
{
   int s = (size < v.size) ? size : v.size;

   if (v.size != size)
      cerr << "copying different size arrays "
           << size << " and " << v.size << "\n";
   for (int i = 0; i < s; ++i)
      p[i] = v.p[i];
   return (*this);
}

vect vect::operator +(const vect& v)
{
   int s = (size < v.size) ? size : v.size;
   vect  sum(s);

   if (v.size != size)
      cerr << "adding different size arrays "
	   << size << " and " << v.size << "\n";
   for (int i = 0; i < s; ++i)
      sum.p[i] = p[i] + v.p[i];
   return (sum);
}

void vect::print()
{
   for (int i = 0; i <= (size-1); ++i)
      cout << p[i] << "\t";
   cout << "\n";
}

void init_vect(vect& v, int start, int incr)
{
   for (int i = 0; i <= v.ub(); ++i) {
      v[i] = start;
      start += incr;
   }
}

main()
{
   vect   a(3), b(3), c(6), d(6);
   int    i, j;

   a[0] = 1 + (a[1] = 1 + ( a[2] = 1)) ;
   init_vect(b, 1, 1);
   init_vect(c, 10, 10);
   init_vect(d, 100, 1);

   cout << "vector a is\n";
   a.print();
   cout << "\nvector b is\n";
   b.print();
   cout << "\nvector c is\n";
   c.print();
   cout << "\nvector d is\n";
   d.print();

   a = b = c;
   a = b + (c = a) + d;
   cout << "vector a is\n";
   a.print();
   cout << "\nvector b is\n";
   b.print();
   cout << "\nvector c is\n";
   c.print();
   cout << "\nvector d is\n";
   d.print();
   return (0);
}
