/*********************************************************************

  Filename:  vect_it.cpp
  Chapter:   7      Templates
  Section:   7.4    Parameterizing the Class vect
  Compiler:  Borland C++     Version 3.1       Summer 1993

  C++ for C Programmers, Edition 2     By Ira Pohl

*********************************************************************/

//Template based safe array type
//  Title: vect_iterator

#include <iostream.h>
#include <stdlib.h>


enum boolean { false, true };

template <class T> class  vect_iterator;
template <class T>
class vect {
public:
  vect();                           //create a size 10 array
  vect(int n);                      //create a size n array
  vect(const vect& v);              //initialization by vect
  vect(const T a[], int n);         //initialization by array
  ~vect() { delete [] p; }           //destroy T's
  int  ub() { return (size - 1); }  //upper bound
  T& operator [](int i) const;            //range checked element
  vect& operator =(const vect& v);
  void print() const
    { int i; for (i = 0; i < size; ++i) cout << p[i] << '\t'; cout << endl;}
private:
  T* p;                             //base pointer
  int size;                         //number of elements
  friend class vect_iterator<T>;
};

template<class T>
class vect_iterator {
public:
  vect_iterator(vect<T>& v):cur_ind(0),pv(&v) {}
  boolean successor();
  boolean predecessor();
  T& item() const { return (pv -> p[cur_ind] ); }
  void reset(int n = 0) {cur_ind = n % pv -> size; }
  int position() const { return (cur_ind); }
private:
  vect<T>* pv;
  int cur_ind;
};

template<class T>
boolean vect_iterator<T>::successor()
{
  if (cur_ind >= pv -> size - 1)
     return (false);
  else {
     ++cur_ind;
     return (true);
  }
}

template<class T>
boolean vect_iterator<T>::predecessor()
{
  if (cur_ind <= 0)
     return (false);
  else {
     --cur_ind;
     return (true);
  }
}

template<class T>
vect<T>::vect()
{
  size = 10;
  p = new T[size];
}

template<class T>
vect<T>::vect(int n)
{
  if (n <= 0) {
     cerr << "illegal vect size: " << n << endl;
     exit(1);
  }
  size = n;
  p = new T[size];
}

template<class T>
vect<T>::vect(const T a[], int n)
{
  if (n < 1) {
     cerr << "illegal vect size: " << n << endl;
     exit(1);
  }
  size = n;
  p = new T[size];
  for (int i = 0; i < size; ++i)
     p[i] = a[i];
}

template<class T>
vect<T>::vect(const vect<T>& v)
{
  size = v.size;
  p = new T[size];
  for (int i = 0; i < size; ++i)
     p[i] = v.p[i];
}

template<class T>
T& vect<T>::operator [](int i) const
{
  if (i < 0 || i > ub()) {
     cerr << "illegal vect index: " << i << endl;
     exit(1);
  }
  return (p[i]);
}

template<class T>
vect<T>& vect<T>::operator =(const vect<T>& v)
{
  int s = (size < v.size) ? size : v.size;

  if (v.size != size)
     cerr << "copying different size arrays "
	  << size << " and " << v.size << endl;
  for (int i = 0; i < s; ++i)
     p[i] = v.p[i];
  return (*this);
}

main()
{
  vect<int> v(8);  vect_iterator<int> a(v);

  for (int i = 0; i < 8; ++i)
    v[i] = i * i;
  do
     cout << a.item() << '\t';
  while (a.successor());
  cout << endl;
  v.print();
  return (0);
}


